#!/usr/local/bin/python

# Post to Blogger
# Copyright (c) 2006 Kesava Yerra. Licensed under BSD.

# Labels work so posting to blogger basically works.

# TODO
# - grab published URL from reply so we can generate a redirect map
# - special-case comments from me for URL

import formatter
import htmllib
import httplib
import urllib
import re
import sys
from xml.sax.saxutils import escape
import feedparser
from transfusion import Transfusion
from util import *

class GoogleLogin:
  def __init__(self, email, passwd, service):
    params = urllib.urlencode({
      'Email': email, 
      'Passwd': passwd,
      'service': service,
      'source': 'persistentInfo-bloggerUpload-1'
    })
    headers = {"Content-type": "application/x-www-form-urlencoded"}

    conn = httplib.HTTPSConnection("www.google.com")
    conn.request("POST", "/accounts/ClientLogin", params, headers)

    response = conn.getresponse()
    data = response.read()

    # If the login fails Google returns 403
    if response.status == 403:
      self._auth_token = None
    else:
      # TODO(mihaip): more reliable splitting/mapping
      self._auth_token = data.splitlines()[2].split('=')[1]

    conn.close()

  def GetAuthToken(self):
    return self._auth_token

class SecurityTokenParser(htmllib.HTMLParser):
  def __init__(self):
    htmllib.HTMLParser.__init__(self, formatter.NullFormatter())

  def parse(self, input):
    self.security_token = None
    self.feed(input)
    
  def do_input(self, attrs):
    if ("name", "securityToken") in attrs:
      for (name, value) in attrs:
        if name == "value":
          self.security_token = value

class BloggerGDataService:
  _BLOGGER_HOST = 'www.blogger.com'
  _BLOGGER_BLOGSPOT_HOST = '%s.blogspot.com'

  _BLOGGER_POST_PATH = '/feeds/%s/posts/default'

  _BLOGGER_ENTRIES_PATH = '/feeds/%s/posts/default'
  _BLOGGER_COMMENTS_PATH = '/feeds/comments/default'
  
  def __init__(self, auth_token, blog_name, blog_id):
    self._auth_token = auth_token
    self._blog_id = blog_id
    self._blog_name = blog_name
  
  def _MakeRequest(self, method, host, path, headers=None, data=None):
    if headers is None:
      headers = {}
      
    if not "Authorization" in headers:
      headers["Authorization"] = "GoogleLogin auth=%s" % self._auth_token

    conn = httplib.HTTPConnection(host)
    conn.request(method, path, data, headers)

    res = conn.getresponse()

    if res.status == 302:
      (host, path) = GetHostAndPath(res.getheader('location'))
      conn.close()
      
      return self._MakeRequest(method, host, path, headers, data)
    else:
      return (conn, res)

  def GetEntries(self):
    return self._GetEntryCollection(
        self._BLOGGER_HOST,
        self._BLOGGER_ENTRIES_PATH % (self._blog_id))
  
  def GetComments(self):
    return self._GetEntryCollection(
      self._BLOGGER_BLOGSPOT_HOST % (self._blog_name),
      self._BLOGGER_COMMENTS_PATH)
  
  def _GetEntryCollection(self, host, path):
    has_next = True
    fetched_entries = []
    
    while has_next:
      # TODO(mihaip): remove to avoid redirect?
      host = "www2.blogger.com"
      (conn, res) = self._MakeRequest("GET", host, path)
      res_text = res.read()
      feed_data = feedparser.parse(res_text)
      conn.close()
      
      print "  Got %d items" % (len(feed_data.entries))
      if len(feed_data.entries) == 0:
        print res_text
      
      fetched_entries.extend(feed_data.entries)
      
      has_next = False
      for link in feed_data.feed.links:
        if link.rel == "next":
          has_next = True
          (host, path) = GetHostAndPath(link.href)
          break
    
    entries = []
    
    for entry in fetched_entries:
      blogger_entry = Entry()
      blogger_entry.FillFromFeedParser(entry)
      entries.append(blogger_entry)
    
    return entries

  def PostEntry(self, entry):
    path = self._BLOGGER_POST_PATH % (self._blog_id)

    headers = {
      "Content-type": "application/atom+xml"
    }
    
    (conn, res) = \
      self._MakeRequest("POST", self._BLOGGER_HOST, path, headers, str(entry))
    feed_data = feedparser.parse(res.read())
    conn.close()
    
    CheckResponse(res, "Could not post entry %s" % entry.title)    
    
    entry.FillFromFeedParser(feed_data.entries[0])
    print "    Posted at %s with post id %s" % \
        (entry.alternate_url, entry.blogger_post_id)
  
  def PostComment(self, entry, comment):
    # First we have to fetch the comment creation form, so we can extract the
    # security token
    blog_id = self._blog_id
    post_id = entry.blogger_post_id
    
    comment_form_url = \
        "https://www2.blogger.com/comment.g?blogID=%s&postID=%s" % (blog_id, post_id)
    
    comment_page = urllib.urlopen(comment_form_url)
    security_token_parser = SecurityTokenParser()
    security_token_parser.parse(comment_page.read())
    comment_page.close()
    security_token = security_token_parser.security_token
    
    # Blogger validates comment author URLs, but Movable Type didn't, so we have
    # to do some checks here
    author_url = comment.author_url or ''
    if author_url:
      (scheme, host, path, query, fragment) = urlparse.urlsplit(author_url)
      # Hostnames are expected to have a TLD
      if not "." in host:
        author_url = ''
    
    # Now we can actually post the comment
    params = urllib.urlencode({
      'blogID': blog_id, 
      'postID': post_id,
      'iden': 'Other',
      'anonName': comment.author_name or '',
      'anonURL': author_url,
      'postBody': comment.content,
      'securityToken': security_token
    })
    
    post_comment_url = "https://www2.blogger.com/comment.do"
    
    success_page = urllib.urlopen(post_comment_url, params)
    response = success_page.read().lower()
    lines = StripHtml(comment.content.replace("&", "&amp;")).split("\n")
    for line in lines:
      line = line.strip().lower()
      words = line.split(" ")
      for word in words:
        if word not in response:
          print response
          raise AssertionError("Did not find comment in response (%s)" % line)
    success_page.close()
    
  
  def DeleteEntry(self, entry):
    (host, path) = GetHostAndPath(entry.self_url)
    (conn, res) = self._MakeRequest("DELETE", host, path)
    
    (status, reason) = (res.status, res.reason)
    conn.close()
    
    CheckResponse(res, "Could not delete entry %s" % entry.id)

  def DeleteComment(self, comment):
    (host, path) = GetHostAndPath(comment.self_url)
    (conn, res) = self._MakeRequest("DELETE", host, path)
    
    (status, reason) = (res.status, res.reason)
    conn.close()
    
    CheckResponse(res, "Could not delete comment %s" % comment.id)

class Entry:
  _BLOGGER_CATEGORY_SCHEME = 'http://www.blogger.com/atom/ns#'
  
  _BLOGGER_ENTRY_ID_RE = re.compile("tag:blogger.com,1999:blog-(\d+).post-(\d+)")
  
  def __init__(self):
    self.id = None
    self.title = None
    self.content = None
    self.published_date = None
    
    self.author_name = None
    self.author_url = None

    self.labels = []
    self.alternate_url = None
    self.self_url = None
    self.edit_url = None
    
    self.is_draft = False
    
    self.comments = []
  
  def FillFromFeedParser(self, entry):
    self.id = entry.id
    self.blogger_post_id = self._BLOGGER_ENTRY_ID_RE.search(entry.id).group(2)
    self.title = entry.title
    self.content = entry.content

    self.labels = []
    if "tags" in entry:
      for tag in entry.tags:
        if tag.scheme == Entry._BLOGGER_CATEGORY_SCHEME:
          self.labels.append(tag.term)
    
    self.alternate_url = GetLinkWithRelation(entry.links, "alternate")
    self.self_url = GetLinkWithRelation(entry.links, "self")
    self.edit_url = GetLinkWithRelation(entry.links, "edit")
  
  def FillFromTransfusion(self, entry):
    self.title = entry["TITLE"].strip()
    self.content = entry["BODY"]
    
    # do linebreak conversion if requested 
    if entry["CONVERT BREAKS"] == "1" or \
       entry["CONVERT BREAKS"] == "__default__":
      self.content = ConvertMTLinebreaks(self.content)
    
    # parse published date
    self.published_date = ParseMTDate(entry["DATE"])
    
    self.labels = (entry["PRIMARY CATEGORY"],)
    
    self.is_draft = entry["STATUS"] == "Draft"
    
    if "COMMENT" in entry:
      for comment in entry["COMMENT"]:
        blogger_comment = Comment()
        blogger_comment.FillFromTransfusion(comment)
        self.comments.append(blogger_comment)

  def __GetLabelElement(self, label):
    return '<category scheme="%s" term="%s"/>' % \
      (self._BLOGGER_CATEGORY_SCHEME, label)

  def __str__(self):
    draft = ''
    
    if self.is_draft:
      draft = """<app:control xmlns:app="http://purl.org/atom/app#">
                <app:draft>yes</app:draft>
            </app:control>"""
    
    published = ''
    updated = ''
    if self.published_date:
      date_str = self.published_date.isoformat()
      published = '<published>%s</published>' % date_str
      updated = '<updated>%s</updated>' % date_str
    
    categories = \
        ''.join([self.__GetLabelElement(label) for label in self.labels])

    author = ''
    if self.author_name or self.author_url:
      author = '<author>'
      if self.author_name:
        author += '<name>%s</name>' % (self.author_name)
      if self.author_url:
        author += '<uri>%s</uri>' % (self.author_url)
      author += '</author>'

    xml_entry = \
      """
      <entry xmlns='http://www.w3.org/2005/Atom'>
        %(draft)s
        %(updated)s
        %(published)s
        <title type='html'>%(title)s</title>
        %(categories)s
        <content type="html">%(content)s</content>
        %(author)s
      </entry>
      """ % {
        'draft': draft,
        'updated': updated,
        'published': published,
        'title': escape(self.title),
        'categories': categories,
        'content': escape(self.content),
        'author': author,
      }
    
    return xml_entry

class Comment(Entry):
  def FillFromTransfusion(self, comment):
    self.author_name = comment["AUTHOR"]
    self.author_url = comment["URL"]
    self.published_date = ParseMTDate(comment["DATE"])
    self.title = '' # TODO(mihaip): generate snippet?
    # We would normally convert MT line breaks, but we don't want to have HTML
    # when posting via Blogger's form
    self.content = comment["TEXT"]
    
def ImportEntries(email, password, blog_name, blog_id):
  gl = GoogleLogin(email, password, 'blogger')
  blogger = BloggerGDataService(gl.GetAuthToken(), blog_name, blog_id)

#   print "Getting existing comments..."
#   comments = blogger.GetComments()
#  
#   print "Deleting existing comments..."  
#   for comment in comments:
#     print "  Deleting \"%s\"..." % comment.title  
#     blogger.DeleteComment(comment)
#    
#   print "Getting existing entries..."
#   entries = blogger.GetEntries()
#    
#   for entry in entries:
#     print entry.alternate_url
# 
#   return
#    
#   print "Deleting existing entries..."
#   for entry in entries:
#     print "  Deleting \"%s\"..." % entry.title
#     blogger.DeleteEntry(entry)
  
  print "Uploading imported entries..."

  (checkpoint_entry_index, checkpoint_entry_id, checkpoint_comment_index) = GetCheckpoint()
  
  entry_index = 0
  
  for (entry, errors) in Transfusion().read(sys.stdin.read()):
    blogger_entry = Entry()
    blogger_entry.FillFromTransfusion(entry)

    # skip over entries before the checkpoint
    if entry_index > checkpoint_entry_index:
      if errors:
        print >> sys.stderr, error
      
      print "  Uploading \"%s\" [%d]..." % (blogger_entry.title, entry_index)
      blogger.PostEntry(blogger_entry)
      
      SetCheckpoint(entry_index, blogger_entry.blogger_post_id, 0)
    
    comment_index = 0
      
    # We have to restore the entry id since it's needed for resuming posting
    # of comments
    if entry_index == checkpoint_entry_index:
      blogger_entry.blogger_post_id = checkpoint_entry_id
    
    for comment in blogger_entry.comments:
      # Skip over comments before the checkpoint
      if entry_index > checkpoint_entry_index or \
         (entry_index == checkpoint_entry_index and 
             comment_index > checkpoint_comment_index):
        print "    Uploading comment [%d/%d]..." % (comment_index + 1, len(blogger_entry.comments))
        blogger.PostComment(blogger_entry, comment)
        
        SetCheckpoint(entry_index, blogger_entry.blogger_post_id, comment_index)
      
      comment_index += 1

    entry_index += 1


if __name__ == '__main__':
  ImportEntries('user.name@gmail.com', 
                'password', 
                # TODO(mihaip): use auto-discovery from blog URL?
                'blog-name', # e.g. "blog-name" for "blog-name.persistent.info"
                'blog-id')