from datetime import datetime
from datetime import timedelta
from datetime import tzinfo
import sgmllib
from time import strptime
import urlparse

def GetHostAndPath(url):
  (scheme, host, path, query, fragment) = urlparse.urlsplit(url)
  if query:
    path += "?" + query
  if fragment:
    fragment += "#" + fragment;
  return (host, path)

def GetLinkWithRelation(links, relation):
  for link in links:
    if link.rel == relation:
      return link.href
  return None

def CheckResponse(res, message):
  if res.status >= 400:
    raise AssertionError("%s\nResponse code %d with reason %s." % 
        (message, res.status, res.reason))

class FixedOffset(tzinfo):
  """Fixed offset in minutes east from UTC."""

  def __init__(self, offset, name):
    self.__offset = timedelta(minutes = offset)
    self.__name = name

  def utcoffset(self, dt):
    return self.__offset

  def tzname(self, dt):
    return self.__name

  def dst(self, dt):
    return ZERO

_DEFAULT_TIMEZONE = FixedOffset(-8 * 60, "PST")

def ParseMTDate(mt_date):
  (year, month, day, hour, minute, second, weekday, julian_day, dst) = \
      strptime(mt_date, "%m/%d/%Y %I:%M:%S %p")
  return \
      datetime(year, month, day, hour, minute, second, 0, _DEFAULT_TIMEZONE)

def ConvertMTLinebreaks(content):
  paragraphs = content.rstrip().split("\n\n")
  content = "<p>" + "</p><p>".join(paragraphs) + "</p>"
  lines = content.split("\n")
  return "<br />".join(lines)

def SetCheckpoint(entry_index, entry_id, comment_index):
  f = file("blogger-upload-checkpoint", "w")
  f.write("%d\n%s\n%d" % (entry_index, entry_id, comment_index))
  f.close()

def GetCheckpoint():
  f = file("blogger-upload-checkpoint", "r")
  [entry_index, entry_id, comment_index] = f.readlines()
  f.close()
  
  return [int(entry_index), entry_id, int(comment_index)]

# From http://flangy.com/dev/python/striphtml.html
class Stripper(sgmllib.SGMLParser):
	def __init__(self):
		sgmllib.SGMLParser.__init__(self)
		
	def strip(self, some_html):
		self.theString = ""
		self.feed(some_html)
		self.close()
		return self.theString
		
	def handle_data(self, data):
		self.theString += data

def StripHtml(text):
  return Stripper().strip(text)